;  _  -**-  _
; (_|-|/\()_\   mon.s: Mini version of the monitor program
;     -**-
; 
; This is the first application program for CHAOS. It's based on the old "chaos.s" downloader/monitor
; program.

; History:
; v1.4:		History added, cut down help, removed LFs.
	
		cpu Z80
		include chaos.i

; Ideas:	disassembler
;		single-step and breakpoints (modify an RST xx to jump to this)

startaddr:	EQU	1900h	; Where we start up pointing to

; --- "Here goes another."

		ORG appbase	; Start of application.

; --- Begin user application code...

monitor:
		LD IX,welcome
		CALL i_pstring

		LD HL,startaddr ; HL contains the current address.

gloop:		LD IX,prompt
		CALL i_pstring
		LD IX,strbuf
		CALL i_gstring
		LD A,(strbuf)	; First character is the command
		CP 'd'
		JR NZ,notd

		; 'd' command: display block
		LD A,L
		AND 11111000b   ; Set the low 8 bits to 0.
		LD L,A
dline:		CALL outadr
		PUSH HL
		LD C,8          ; C-pos counter  (8 positions)
ddat:		LD A,(HL)
		CALL outhex
		INC HL
		DEC C
		JR NZ,ddat
		POP HL
		CALL outcr
		JP gloop

notd:		CP 'a'
		JR NZ,nota
		; 'a' command: set address
		LD IX,strbuf+2
		CALL i_gethex
		LD H,A
		LD IX,strbuf+4
		CALL i_gethex
		LD L,A
		JP gloop

nota:		CP 'R'
		JR NZ,notr

		; 'R' command: reboot.
		RST 00h
		; We'll never get here.

notr:		CP 'G'
		JR NZ,notg

		; 'G' command: run from this address
		JP (HL)
		; We'll never get here.

notg:		CP 'h'
		JR NZ,noth

		; 'h' command: help
		LD IX,help
		CALL i_pstring
		JP gloop

noth:		CP 'm'
		JR NZ,notm

		; 'm' command: input
		LD IX,strbuf+2
		CALL i_gethex
		LD D,A
		LD A,D
		LD (HL),A
		CALL outadr
		LD IX,colon
		CALL i_pstring
		LD A,(HL)
		CALL outhex
		INC HL
		CALL outcr
		JP gloop

notm:		CP 'r'
		JR NZ,notrr
		; 'r' - receive data
		LD IX,nimp
		CALL i_pstring
		JP gloop

notrr:		CP 'i'
		JR NZ,noti
		; 'i' - in from a port
		LD IX,strbuf+2
		CALL i_gethex
		LD C,A
		IN A,(C)
		LD C,A
		CALL outcr
		LD A,C
		CALL outhex
		CALL outcr
		JP gloop

noti:		CP 'o'
		JR NZ,noto
		; 'o' - out port, value
		LD IX,strbuf+2
		CALL i_gethex
		LD C,A
		LD IX,strbuf+5
		CALL i_gethex
		OUT (C),A
		JP gloop

noto:		CP 'c'
		JR NZ,notc
		; 'c' - display ASCII code
		LD IX,hitakey
		CALL i_pstring
		CALL i_in
		LD C,A
		CALL i_out
		CALL outsp
		LD A,C
		CALL outhex
		CALL outcr
		JP gloop

notc:		CP 'x'
		JR NZ,notx
		; 'x' - exit the monitor
		RET
	
notx:		CP 'l'
		JR NZ,notl
		LD IX,strbuf+2
		PUSH HL
		CALL i_fread
		POP HL
		LD A,B
		CALL outhex
		LD A,C
		CALL outhex
		LD IX,bread
		CALL i_pstring
		JP gloop

; w 0123 filename
notl:		CP 'w'
		JR NZ,notw
		LD IX,strbuf+2
		CALL i_gethex
		LD B,A
		LD IX,strbuf+4
		CALL i_gethex
		LD C,A	
		LD IX,strbuf+7	
		PUSH HL
		LD A,'x'	; def type executable - not ideal.
		CALL i_fwrite
		POP HL
		JP gloop

notw:

nocmd:		LD IX,error
		CALL i_pstring
		JP gloop

; --- Routines

outadr:		LD A,H
		CALL outhex
		LD A,L
		CALL outhex
		RET

outsp:		PUSH AF
		LD A,' '
		CALL i_out
		POP AF
		RET

outcr:		PUSH AF
		LD A,cr
		CALL i_out
		POP AF
		RET

outhex:		PUSH AF
		PUSH IX
		LD IX,hexbuf
		CALL i_fhex
		LD IX,hexbuf
		CALL i_pstring
		POP IX
		POP AF
		RET

; --- Data area

hexbuf:		DB "xx",eos
strbuf:		DS 80
		DB eos
welcome:	DB "Mon 1.4 OK, h=help",cr,eos
prompt:		DB "# ",eos
error:		DB "Unknown command!",cr,eos
nimp:		DB "Not implemented yet.",cr,eos
bread:		DB " bytes read.",cr,eos
colon:		DB ": ",eos
help:		DB cr
;		DB "Mon: Addr Val Port File Len",cr
;		DB "a <a>     set addr",cr
;		DB "d         dump block",cr
;		DB "R         restart 00h",cr
;		DB "G         go",cr
;		DB "h         help",cr
;		DB "m <v>     input data",cr
;		DB "r         receive data",cr
;		DB "i <p>     input from port",cr
;		DB "o <p> <v> output to port",cr
;		DB "c         display ASCII code",cr
;		DB "l <f>     load file to current address",cr
;		DB "w <l> <f> write current address to file",cr
;		DB "x         exit",cr
		DB "Addr Val Pt Fl Len",cr
		DB "aA  addr",cr
		DB "d   dump",cr
		DB "R   reboot",cr
		DB "G   go",cr
		DB "h   help",cr
		DB "mV  write mem",cr
		DB "iP  port in",cr
		DB "oPV port out",cr
		DB "c   ASCII code",cr
		DB "lF  load file",cr
		DB "wLF write file",cr
		DB "x   exit",cr
		DB eos
hitakey:	DB "key...",cr,eos
		
		END
