; f.s - 16-bit minimalist False interpreter		Adam Sampson, 1998

; Differences:
;	Stack is 16-bit (Amiga False is 32-bit, Portable False is "int").
;	While works with one clause: (.....)#, which always gets executed
;	 the first time through the loop. (It doesn't look for a value on
;	 the stack beforehand.) It would be trivial to make while do this;
;	 but then, we wouldn't need if, would we? :)
;	Uses () rather than [] (although the change would be trivial).
;	No explicit comments. (comment)% does it.
;	No inline assembly. (Obviously.)
;	: is limited to the exec space (so it doesn't damage the OS). Well, when it works it will be.
;	Pick is O rather than O/.
;	Flush (beta) doesn't exist.
;	< exists.

; False is 0, true is anything else.
		
	cpu Z80
	include chaos.i
       	ORG appbase	; Start of application.
	
	INC IX
	INC IX
	LD HL,membuf
	CALL i_fread
	LD A,B
	OR C
	JR NZ,go
	LD IX,nofile
	CALL i_pstring
	RET
nofile:	DB "?file",cr,lf,eos
go:	LD IY,membuf+memlen-1	; initial DSP (data SP)
	LD IX,membuf	; initial PC
donext:	LD A,(IX+0)
	NOP
	LD HL,numtab
	CALL inlist
	JR NZ,cnotnum
	CALL donumber
	JR donext
cnotnum:
	LD HL,vartab
	CALL inlist
	JR NZ,cnotvar
	CALL dovariable
	JR donext
cnotvar:	
	CP 'X'
	RET Z		; finish if we hit an X
	LD HL,cmdtab
findcmd:
	CP (HL)		; found it in the table?
	JR Z,foundcmd	; yes, do something with it.
	LD E,A
	LD A,(HL)
	OR A		; optimised cp 0h
	JR Z,nextcmd	; if it's a null, give up and get the next one
	LD A,E
	INC HL
	INC HL
	INC HL
	JR findcmd
	
foundcmd:
	PUSH IX
	PUSH HL
	POP IX
	LD L,(IX+2)
	LD H,(IX+1)
	POP IX
	CALL jumphl	
nextcmd:
	INC IX
	JP donext
	RET

jumphl:	JP (HL)

; Basic scheme of ops:
;	:;	poke, peek
;	+-/*	add, sub, div, mul
;	=<>	equals, *lt, *gt	
;	&|~	*and, *or, not
;	TF	true, false
;	$%\@O	dup, kill, swap, rot, *pick
;	!?#	call, if, *while
;	^.,	getchar, putnum, putchar
;	"string" putstring
;	[0-9]'	insertnum, quotechar
;	(code)	lambda code	
		
cmdtab:	DB '+',cadd/256,cadd&255
	DB '-',csub/256,csub&255
	DB '*',cmul/256,cmul&255
	DB '/',cdiv/256,cdiv&255
	DB '$',duplicate/256,duplicate&255
	DB '%',off/256,off&255
	DB 'T',pushtrue/256,pushtrue&255
	DB 'F',pushfalse/256,pushfalse&255
	DB ':',cpoke/256,cpoke&255
	DB ';',cpeek/256,cpeek&255
	DB '\\',swap/256,swap&255
	DB '@',rot/256,rot&255
	DB '=',equal/256,equal&255
	DB ',',cout/256,cout&255
	DB '.',outnum/256,outnum&255
	DB '^',cin/256,cin&255
	DB '~',cnot/256,cnot&255
	DB '(',leftie/256,leftie&255
	DB ')',rightie/256,rightie&255
	DB '!',bang/256,bang&255
	DB '\"',string/256,string&255
	DB '?',cif/256,cif&255
	DB '#',cwhile/256,cwhile&255
	DB 27h,quotechar/256,quotechar&255 ;'
	DB 0,0,0

quotechar:
	INC IX
	LD A,(IX+0)
	LD L,A
	LD H,0h
	JR dojpon
	
cpoke:	CALL off	;:	
	EX DE,HL
	CALL off
	EX DE,HL
	PUSH IX
	PUSH HL
	POP IX
	NOP
	LD (IX+0),E
	LD (IX+1),D
	POP IX
	RET

cpeek:	CALL off	;;
	PUSH IX
	PUSH HL
	POP IX
	NOP
	LD L,(IX+0)
	LD H,(IX+1)
	POP IX
	JR dojpon
	
cadd:	CALL off	;+
	EX DE,HL
	CALL off
	ADD HL,DE
dojpon:	JP on

csub:	CALL off	;-
	EX DE,HL
	CALL off
	SCF
	CCF
	SBC HL,DE
	JR dojpon

; Unashamedly nicked from maths.z in the tetris game for z81.
cmul:	CALL off	;*
	EX DE,HL
	CALL off
	; begin borrowed code
	ld b,h
	ld c,l
	ld hl,0
	ld a,16
mulp:	and a
	rr d
	rr e
	jr nc,musp
	add hl,bc
musp:	and a
	rl c
	rl b
	dec a
	jr nz,mulp
	; end borrowed code
	JR dojpon

cdiv:	CALL off	;/
	EX DE,HL
	CALL off
	CALL divide
	JR dojpon
		
; as doesn't like the label "cdup" for some reason.
duplicate:	
	CALL off	;$
	CALL on
	JR dojpon

swap:	CALL off	;\
	EX DE,HL
	CALL off
	EX DE,HL
	CALL on
	EX DE,HL
	JR dojpon

rot:	CALL off	;@ (a b c - b c a) 
	PUSH HL
	CALL off
	EX DE,HL
	CALL off
	EX DE,HL
	CALL on
	EX DE,HL
	CALL on
	POP HL
	JR dojpon
	
equal:	CALL off	;=
	EX DE,HL
	CALL off
	LD A,D
	CP H
	JR NZ,pushfalse
	LD A,E
	CP L
	JR NZ,pushfalse
pushtrue:		;T
	LD HL,0FFFFh
	JR dojpon
	
pushfalse:		;F
	LD HL,0h
	JP on
		
cout:	CALL off	;,
	LD A,L
	JP i_out

cin:	CALL i_in	;^
	LD H,0h
	LD L,A
	JP on

cnot:	CALL off	;~
	LD A,H
	CPL
	LD H,A
	LD A,L
	CPL
	LD L,A
	JP on
			
leftie:	PUSH IX		;(
	POP HL
	INC HL
	CALL on
	LD C,1h
contscan:	
	INC IX
	LD A,(IX+0)
	CP '('
	CALL Z,incc
	CP ')'
	JR NZ,contscan
	DEC C
	JR NZ,contscan
	RET
incc:	INC C
	RET

; when we're in a while, the PS looks like:	 (->top)
; <continue ret address> <do again address> <0000>
; otherwise, it looks like
; <ret address>
rightie:
	NOP
	CALL poppsp	;)
	LD A,H
	OR L
	JR Z,rightwhile
	PUSH HL
	POP IX
	RET

rightwhile:
	CALL off
	LD A,H
	OR L
	JR NZ,doagain
	CALL poppsp	; drops the original call address
	JR rightie	; now return to the original routine
doagain:
	CALL poppsp	; get the "do again" address
	CALL pushpsp	; put it back
	PUSH HL		; push the 0h for us to find next time
	LD HL,0h
	CALL pushpsp
	POP IX		; return to the do again address
	RET

cwhile:	PUSH IX		;#
	POP HL
	CALL pushpsp
	CALL off
	DEC HL
	PUSH HL
	CALL pushpsp
	LD HL,0h
	CALL pushpsp
	POP IX
	RET

pushpsp:
	LD IX,(psp)
	LD (IX+0),H
	LD (IX+1),L
	INC IX
	INC IX
	LD (psp),IX	
	RET	

poppsp:	LD IX,(psp)
	DEC IX
	DEC IX
	LD H,(IX+0)
	LD L,(IX+1)
	LD (psp),IX
	RET
	
cif:	CALL swap	;?
	CALL off
	LD A,H
	OR L
	JP Z,off
; --------fall through---------			
bang:	PUSH IX		;!
	POP HL
	CALL pushpsp
	CALL off
	PUSH HL
	POP IX
	DEC IX
	RET

string:	INC IX
stringl:	
	LD A,(IX+0)
	NOP
	CP 34
	RET Z
	CALL i_out
	INC IX
	JR stringl
		
donumber:		;0-9
	LD H,0h
	LD A,(IX+0)
	INC IX
	SUB '0'
	LD L,A
donextnum:
	LD A,(IX+0)
	PUSH HL
	LD HL,numtab
	CALL inlist
	POP HL
	JP NZ,on	; and thus return
	CALL multen
	LD B,0h
	SUB '0'
	LD C,A
	ADD HL,BC
	INC IX
	JR donextnum

; if the char in A is in list HL, return Z else return NZ
inlist:
	CP (HL)
	RET Z		; ret with Z
	LD B,A
	LD A,(HL)
	CP 0h
	JR Z,notnum
	LD A,B
	INC HL
	JR inlist
notnum:	CP 1h		; which it can't be, as it's 0h
	LD A,B
	RET		; ret with NZ

numtab:	DB "0123456789",0h
	
; multiply the number in HL by ten
multen:	PUSH HL
	POP BC
	LD E,09h
multen0:
	ADD HL,BC
	DEC E
	JR NZ,multen0
	RET			

; push and pop, IY is DSP, HL is data
on:	LD (IY+1),H
	LD (IY+0),L
	DEC IY
	DEC IY
	RET

off:	INC IY
	INC IY
	LD H,(IY+1)
	LD L,(IY+0)
	RET

; From z81 tetris math.z
;divide, hl=hl/de, de=hl%de
;af/bc corrupt
dividey:
	dw 0
divide:
	ld b,h
	ld c,l
	;see if we're trying to divide by zero
	ld a,d
	or e
	ret z
	ld (dividey),de
	ld de,0
	ld hl,0
	ld a,16
dvlp:	push af
	and a
	rl l
	rl h
	and a
	rl e
	rl d
	bit 7,b	
	jr z,dvs1
	ld a,1
	or l
	ld l,a
dvs1:	push hl
	and a
	push de
	ld de,(dividey)
	sbc hl,de
	pop de
	jp m,dvs2
	ex (sp),hl
	ld a,1
	or e
	ld e,a
dvs2:	pop hl
	and a
	rl c
	rl b
	pop af
	dec a
	jr nz,dvlp
	ex de,hl
	ret

outnum:	CALL off
	EX DE,HL
	CALL itoa
	PUSH IX
	LD IX,numtmp
	CALL i_pstring
	POP IX
	RET
	
;convert number in de to ascii in internal buffer
;entry: de=number
;exit:  hl=addr of number in internal buffer, NUL terminated
itoa:	ld hl,numtmp
	ld c,numtmpend-numtmp-1
itoa0:	ld a,' '
	ld (hl),a
	inc hl
	dec c
	jr nz,itoa0
	ld b,10		; Base, in theory.
	ld hl,numtmpend-1
	xor a
	ld (hl),a
dispnlp:
	push bc
	push hl
	ex de,hl
	ld e,b
	ld d,0
	call divide
	ld a,e
	add a,'0'	; this ain't no ZX81!
	ex de,hl	; so de now is result of division
	pop hl
	pop bc
	dec hl
	ld (hl),a
	ld a,d
	or e
	jp nz,dispnlp
	ret

dovariable:
	LD A,(IX+0)
	SUB 'a'
	SLA A
	LD C,A
	LD B,0h
	LD HL,varbuf
	ADD HL,BC
	CALL on
	INC IX
	RET
	
vartab:	DB "abcdef",0h
vartabend:	
numtmp: DB "      ",eos		; buffer for number output
numtmpend:	
psp:	DB progstack/256,progstack&255	; program stack pointer
varbuf:				; space for variables
progstack: EQU varbuf+(2*(vartabend-vartab-1))	; space for the program stack
membuf:	EQU progstack+40	; put the buffer at the end of the program.
memlen:	EQU 400h
	END
