; AZZ80 console driver. This provides a console device (0,0).

; INCL dragonkb.s - za80 can't nest includes! (b*(*(*(**^%^%"!)
; Dragon keyboard reader.

; Set up the local 8255 for the keyboard scanner.
k_init:			
		CALL k_rmain		; Read the main section by default.
		RET

 ; Select the mode to read the keyboard in, i.e. which set of lines
; are tristated.

k_rmain:
		PUSH AF
		LD A,10001011b		; A(4)=out, B(1)=in, Ch(3)=in, Cl(0)=in
		OUT (43h),A
		POP AF
		RET

k_rshift:
		PUSH AF
		LD A,10011010b		; A(4)=in, B(1)=in, Ch(3)=in, Cl(0)=out
		OUT (43h),A
		POP AF
		RET

; k_getraw	Get a raw keycode, K_no_key if no key down.

k_getraw:
		CALL k_rmain
		PUSH BC
k_gr0:	LD B,7h
		LD C,1h
k_gr2:	LD A,C
		OUT (40h),A
		IN A,(41h)
		CP 0h
		JR NZ,k_gr1
		RLC C
		DEC B
		JR NZ,k_gr2
		; no key is down.
		LD A,K_no_key			; "no key down".
		JR k_grend
k_gr1:	; got a key, col# in B, row bits in A.
		CALL k_debounce		; debounce.
		IN A,(41h)
		PUSH BC
		LD C,A
		LD B,7h
		LD A,1h
k_gr4:	CP C
		JR Z,k_gr3
		DEC B
		RLC A
		CP 01h
		JR NZ,k_gr4
		LD A,K_bad_key		; "invalid or rollover"
		POP BC
		JR k_grend
k_gr3:	; worked out row to B.
		LD A,B
		RLC A
		RLC A
		RLC A
		POP BC
		OR B
		CPL
		AND 00111111b			; format now %00rrrccc - see DragonKB
k_grend:
		POP BC
		RET

; Get the "shift" row of the keyboard.
k_getshift:
		CALL k_rshift
		LD A,01h
		OUT (42h),A
		IN A,(41h)
; We've got the bits; now, work out the key number. It is assumed that the
; user will never want to use shift-enter, shift-clear or shift-break.
		PUSH BC
		LD C,A
		LD B,7h
		LD A,1h
k_gs4:	CP C
		JR Z,k_gs5			; got it!
		DEC B
		RLC A
		CP 01h
		JR NZ,k_gs4
		LD A,K_no_key		; no shift key down
		JR k_gse
k_gs5:	LD A,B
k_gse:	POP BC
		CALL k_rmain
		RET

; Wait for and get a raw keypress. Mains take priority over shifts.
k_waitraw:
; Wait for the main keys to clear.
k_wr_1:	CALL k_getraw
		CP K_no_key
		JR NZ,k_wr_1
; Wait for the "non-shift shift" keys to clear.
k_wr_1a:
		CALL k_getshift
		CP K_no_key
		JR Z,k_wr_1b
		CP K_shift
		JR NZ,k_wr_1a
k_wr_1b:
		CALL k_prebounce	; Debounce.
; First, check the main keys...
k_wr_0:	CALL k_getraw
		CP K_no_key
		JR Z,k_wr_2			; No key? Check the shifts.
		CP K_bad_key
		JR Z,k_wr_1
		JR k_wr_e			; Got one!
; then check the shifts.
k_wr_2:	CALL k_getshift
		CP K_no_key
		JR Z,k_wr_0			; No key? Check the mains.
		CP K_shift
		JR Z,k_wr_0			; Shift? Ignore it.
		ADD A,40h				; Add 40h to it, to take it out of the main range.		
k_wr_e:	RET

; Decode a raw keypress.
k_cookraw:
		PUSH HL
		PUSH BC
		LD HL,k_keymap
		LD B,0h
		LD C,A
		ADD HL,BC
		LD A,(HL)
		POP BC
		POP HL
		RET

; Decode a raw shifted keypress.
k_cooksraw:
		PUSH HL
		PUSH BC
		LD HL,k_keymaps
		LD B,0h
		LD C,A
		ADD HL,BC
		LD A,(HL)
		POP BC
		POP HL
		RET

; Get a keypress and return its ASCII code. Handle the shift key.
k_getkey:
		CALL k_waitraw		; no, it's not.
		PUSH AF
		CALL k_rshift		; Set to read-shift mode.
		LD A,01h
		OUT (42h),A
		IN A,(41h)
		AND 80h
		JR NZ,k_gkshf		; if a shift key is down...
		POP AF
		CALL k_cookraw
		JR k_gke
k_gkshf:
		POP AF
		CALL k_cooksraw
k_gke:	RET

; Short delays for debouncing purposes.
k_debounce:
		PUSH AF
		PUSH HL
		LD HL,K_deb_lenA
k_db_0:	DEC HL
		LD A,H
		OR L
		JR NZ,k_db_0
		POP HL
		POP AF
		RET

k_prebounce:
		PUSH AF
		PUSH HL
		LD HL,K_deb_lenB
k_pb_0:	DEC HL
		LD A,H
		OR L
		JR NZ,k_pb_0
		POP HL
		POP AF
		RET

; Keymap for the Dragon keyboard.
k_keymap:
row0:	DB "08@hpx"
		DB 13, 0		; enter
row1:	DB "19aiqy"
		DB 12, 0		; clear
row2:	DB "2:bjrz"
		DB 3, 0		; break
row3:	DB "3;cks"
		DB 4, 0, 0	; up
row4:	DB "4,dlt"
		DB 5, 0, 0	; down
row5:	DB "5-emu"
		DB 8, 0, 0	; left
row6:	DB "6.fnv"
		DB 9, 0, 0	; right
row7:	DB "7/gow "
		DB 0, 0		; "old shift"
rowsh:	DB "~"		; "new shift" -- this should not come up
		DB 0, 0, 0, 0	; spare
		DB 3, 12, 13	; break, clear, return

; Shifted keytops.
k_keymaps:
srow0:	DB "0(@HPX"
		DB 13, 0		; enter
srow1:	DB "!)AIQY"
		DB 12, 0		; clear
srow2:	DB 34			; quote
		DB "*BJRZ"
		DB 3, 0		; break
srow3:	DB "#+CKS"
		DB 4, 0, 0	; up
srow4:	DB "$<DLT"
		DB 5, 0, 0	; down
srow5:	DB "%=EMU"
		DB 8, 0, 0	; left
srow6:	DB "&>FNV"
		DB 9, 0, 0	; right
srow7:	DB "'?GOW "
		DB 0, 0		; "old shift"
srowsh:	DB "~"		; "new shift" -- this should not come up
		DB 0, 0, 0, 0	; spare
		DB 3, 12, 13	; break, clear, return

; INCL a80video.s
; *** AZZ80 video routines.

a80vidinit:
			LD HL,0h
s_init_vc_loop:		
			LD A,H
			OUT (61h),A
			LD A,L
			OUT (60h),A
			LD A,0h
			OUT (62h),A			
			INC HL
			LD A,H
			CP 20h
			JR NZ,s_init_vc_loop
			RET

; A=ascii
vidout:		PUSH HL
			PUSH BC
			LD HL,v_convtab
			LD B,0h
			LD C,A
			ADD HL,BC
			LD A,(HL)
			POP BC
			POP HL
			PUSH AF
			CALL vidout_docheck
			LD A,(v_xcoord)
			INC A
			LD (v_xcoord),A
			CALL vidout_docheck
			LD A,(v_xcoord)
			LD H,A
			LD A,(v_ycoord)
			LD L,A
			POP AF
			CP 255
			JR Z,vidout_ret
			CP 254
			JR Z,vidout_del
			JP v_char

vidout_ret:	LD A,25
			LD (v_xcoord),A
			JP vidout_docheck

vidout_del:	LD A,(v_xcoord)
			DEC A
			LD (v_xcoord),A
			CALL vidout_docheck
			LD A,(v_xcoord)
			LD H,A
			LD A,(v_ycoord)
			LD L,A
			LD A,36
			CALL v_char
			LD A,(v_xcoord)
			DEC A
			LD (v_xcoord),A
			JP vidout_docheck

; handle wraparounds
vidout_docheck:
			PUSH AF
			LD A,(v_xcoord)
			CP 25
			JR NZ,vidout_docheck_next
			PUSH HL
			LD A,(v_ycoord)
			INC A
			LD (v_ycoord),A
			CALL vidout_clearline
			POP HL
			LD A,0h
			LD (v_xcoord),A
vidout_docheck_next:
			LD A,(v_ycoord)
			CP 16
			JR NZ,vidout_docheck_done
			LD A,0h
			LD (v_ycoord),A
			LD (v_xcoord),A
			CALL vidout_clearline
vidout_docheck_done:
			POP AF
			RET

vidout_clearline:
			LD L,A
			LD H,0h
vidout_docheck_clr:
			LD A,36
			CALL v_char
			INC H
			LD A,H
			CP 31
			JR NZ,vidout_docheck_clr
			RET

; H=X, L=Y, A=char#
v_char:		PUSH HL
			PUSH IX
			PUSH BC
			PUSH DE
			PUSH AF
			INC H		; to get on to the screen
			INC H
			INC L
			INC L
			LD A,H
			LD H,0h
			ADD HL,HL	;x2
			ADD HL,HL	;x4
			ADD HL,HL	;x8
			ADD HL,HL	;x16
			ADD HL,HL	;x32.
			PUSH HL
			POP BC		; x6 to turn into a line address
			ADD HL,BC
			ADD HL,BC
			ADD HL,BC
			ADD HL,BC
			ADD HL,BC
			ADD HL,BC
			ADD HL,BC
			ADD HL,BC
			ADD HL,BC
			ADD HL,BC
			ADD HL,BC
			LD B,0h
			LD C,A
			ADD HL,BC
			; HL now contains the base address.
			POP AF
			PUSH AF
			LD IX,v_chartab
			LD C,A
			LD B,0h
			LD A,06h
vchar_l1:	ADD IX,BC
			DEC A
			JR NZ,vchar_l1
			; IX now points at the character.
			LD E,06h
			LD BC,0020h	; 32 in decimal
vchar_l2:	LD A,H
			OUT (61h),A	; set the address registers
			LD A,L
			OUT (60h),A
			LD A,(IX+0)
			OUT (62h),A
			ADD HL,BC
			LD A,H
			OUT (61h),A	; set the address registers
			LD A,L
			OUT (60h),A
			LD A,(IX+0)
			OUT (62h),A
			ADD HL,BC
			INC IX
			DEC E
			JR NZ,vchar_l2
			POP AF
			POP DE
			POP BC
			POP IX
			POP HL
			RET

; characters
v_chartab:

; uppercase
DB 00110000b ;0
DB 11001100b
DB 11111100b
DB 11001100b
DB 11001100b
DB 00000000b

DB 00111100b ;1
DB 11001100b
DB 00111100b
DB 11001100b
DB 00111100b
DB 00000000b

DB 11110000b ;2
DB 00001100b
DB 00001100b
DB 00001100b
DB 11110000b
DB 00000000b

DB 00111100b ;3
DB 11001100b
DB 11001100b
DB 11001100b
DB 00111100b
DB 00000000b

DB 11111100b ;4
DB 00001100b
DB 11111100b
DB 00001100b
DB 11111100b
DB 00000000b

DB 11111100b ;5
DB 00001100b
DB 11111100b
DB 00001100b
DB 00001100b
DB 00000000b

DB 11110000b ;6
DB 00001100b
DB 00001100b
DB 11001100b
DB 11110000b
DB 00000000b

DB 11001100b ;7
DB 11001100b
DB 11111100b
DB 11001100b
DB 11001100b
DB 00000000b

DB 00110000b ;8
DB 00110000b
DB 00110000b
DB 00110000b
DB 00110000b
DB 00000000b

DB 11111100b ;9
DB 11000000b
DB 11000000b
DB 11001100b
DB 11110000b
DB 00000000b

DB 11001100b ;10
DB 11001100b
DB 00111100b
DB 11001100b
DB 11001100b
DB 00000000b

DB 00001100b ;11
DB 00001100b
DB 00001100b
DB 00001100b
DB 11111100b
DB 00000000b

DB 11001100b ;12
DB 11111100b
DB 11001100b
DB 11001100b
DB 11001100b
DB 00000000b

DB 00111100b ;13
DB 11001100b
DB 11001100b
DB 11001100b
DB 11001100b
DB 00000000b

DB 00110000b ;14
DB 11001100b
DB 11001100b
DB 11001100b
DB 00110000b
DB 00000000b

DB 00111100b ;15
DB 11001100b
DB 00111100b
DB 00001100b
DB 00001100b
DB 00000000b

DB 00110000b ;16
DB 11001100b
DB 11001100b
DB 00111100b
DB 11110000b
DB 00000000b

DB 00111100b ;17
DB 11001100b
DB 00111100b
DB 11001100b
DB 11001100b
DB 00000000b

DB 11110000b ;18
DB 00001100b
DB 11111100b
DB 11000000b
DB 00111100b
DB 00000000b

DB 11111100b ;19
DB 00110000b
DB 00110000b
DB 00110000b
DB 00110000b
DB 00000000b

DB 11001100b ;20
DB 11001100b
DB 11001100b
DB 11001100b
DB 00110000b
DB 00000000b

DB 11001100b ;21
DB 11001100b
DB 11001100b
DB 11110000b
DB 11000000b
DB 00000000b

DB 11001100b ;22
DB 11001100b
DB 11001100b
DB 11111100b
DB 11001100b
DB 00000000b

DB 11001100b ;23
DB 11001100b
DB 00110000b
DB 11001100b
DB 11001100b
DB 00000000b

DB 11001100b ;24
DB 11001100b
DB 00110000b
DB 00110000b
DB 00110000b
DB 00000000b

DB 11111100b ;25
DB 11000000b
DB 00110000b
DB 00001100b
DB 11111100b
DB 00000000b

; numbers

DB 11111100b ;26
DB 11001100b
DB 11001100b
DB 11001100b
DB 11111100b
DB 00000000b

DB 00110000b ;27
DB 00111100b
DB 00110000b
DB 00110000b
DB 00110000b
DB 00000000b

DB 00111100b ;28
DB 11000000b
DB 00110000b
DB 00001100b
DB 11111100b
DB 00000000b

DB 00111100b ;29
DB 11000000b
DB 00111100b
DB 11000000b
DB 00111100b
DB 00000000b

DB 11001100b ;30
DB 11001100b
DB 11110000b
DB 11000000b
DB 11000000b
DB 00000000b

DB 11111100b ;31
DB 00001100b
DB 00111100b
DB 11000000b
DB 00111100b
DB 00000000b

DB 11110000b ;32
DB 00001100b
DB 00111100b
DB 11001100b
DB 00111100b
DB 00000000b

DB 11111100b ;33
DB 11000000b
DB 00110000b
DB 00001100b
DB 00001100b
DB 00000000b

DB 00110000b ;34
DB 11001100b
DB 00110000b
DB 11001100b
DB 00110000b
DB 00000000b

DB 11110000b ;35
DB 11001100b
DB 11110000b
DB 11000000b
DB 00110000b
DB 00000000b

; punctuation

DB 00000000b ;36 (default for non-printables)
DB 00000000b
DB 00000000b
DB 00000000b
DB 00000000b
DB 00000000b

DB 00110000b ;37
DB 00110000b
DB 00110000b
DB 00110000b
DB 00000000b
DB 00110000b

DB 11001100b ;38
DB 11001100b
DB 00000000b
DB 00000000b
DB 00000000b
DB 00000000b

DB 00110000b ;39 (a hash, honest)
DB 11111100b
DB 00110000b
DB 11111100b
DB 00110000b
DB 00000000b

DB 00110000b ;40
DB 00110000b
DB 00000000b
DB 00000000b
DB 00000000b
DB 00000000b

DB 00110000b ;41
DB 00001100b
DB 00001100b
DB 00001100b
DB 00110000b
DB 00000000b

DB 00001100b ;42
DB 00110000b
DB 00110000b
DB 00110000b
DB 00001100b
DB 00000000b

DB 11001100b ;43 (star)
DB 00110000b
DB 11001100b
DB 00000000b
DB 00000000b
DB 00000000b

DB 00000000b ;44 (plus)
DB 00110000b
DB 11111100b
DB 00110000b
DB 00000000b
DB 00000000b

DB 00000000b ;45 (comma)
DB 00000000b
DB 00000000b
DB 00000000b
DB 00110000b
DB 00001100b

DB 00000000b ;46 (comma)
DB 00000000b
DB 00000000b
DB 00000000b
DB 00110000b
DB 00000000b

DB 00000000b ;47
DB 00000000b
DB 00000000b
DB 11111100b
DB 00000000b
DB 00000000b

DB 11000000b ;48
DB 00110000b
DB 00110000b
DB 00001100b
DB 00001100b
DB 00000011b

DB 00000000b ;49
DB 00000000b
DB 00110000b
DB 00000000b
DB 00110000b
DB 00000000b

DB 00000000b ;50
DB 00000000b
DB 00110000b
DB 00000000b
DB 00110000b
DB 00001100b

DB 11000000b ;51
DB 00110000b
DB 00001100b
DB 00110000b
DB 11000000b
DB 00000000b

DB 00001100b ;52
DB 00110000b
DB 11000000b
DB 00110000b
DB 00001100b
DB 00000000b

DB 00110000b ;53
DB 11001100b
DB 11000000b
DB 00110000b
DB 00000000b
DB 00110000b

DB 00111100b ;54 (vaguely @)
DB 11000011b
DB 11110011b
DB 11110011b
DB 11110011b
DB 00111100b

DB 00000000b ;55
DB 00000000b
DB 11111100b
DB 00000000b
DB 11111100b
DB 00000000b

DB 11111100b ;56
DB 00001100b
DB 00001100b
DB 00001100b
DB 11111100b
DB 00000000b

DB 11111100b ;57
DB 11000000b
DB 11000000b
DB 11000000b
DB 11111100b
DB 00000000b

DB 00000000b ;58
DB 00000000b
DB 00000000b
DB 00000000b
DB 00000000b
DB 11111111b

; ASCII-to-char conv table
v_convtab:	DB 36,36,36,36,36,36,36,36,36,36,36,36,36,255,36,36 ; first 32 non-printable
			DB 36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36
			; <spc>!"#$%&'()*+,-./
			DB 36,37,38,39,36,36,36,40,41,42,43,44,45,47,46,48 ; 32-47 punctuation
			; 0123456789
			DB 26,27,28,29,30,31,32,33,34,35					; 48-57 numbers
			; :;<=>?@										
			DB 49,50,51,55,52,53,54								; 58-64 punctuation
			; ABCDEFGHIJKLMNOPQRSTUVWXYZ
			DB 0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19 ; 65-90 capitals
			DB 20,21,22,23,24,25
			; [\]^_`
			DB 56,36,57,36,58,36								; 91-96 punctuation
			; abcdefghijlmnopqrstuvwxyz
			DB 0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19 ; 97-122 lowercase
			DB 20,21,22,23,24,25
			; {|}~<del>
			DB 36,36,36,36,36									; 123-127 punctuation

a80coninit:	XOR A
			LD (v_xcoord),A
			LD (v_ycoord),A
			CALL a80vidinit
			CALL k_init
			RET

a80conout:	LD A,C
			JP vidout

a80conin:	JP k_getkey

a80const:	CALL k_getraw
			CP K_no_key
			JR NZ,a80const_0
			LD A,0h
			RET
a80const_0:	LD A,0FFh
            RET

; Provide device 0.
ttyinit:	EQU a80coninit
ttyout:		EQU a80conout
ttyin:		EQU a80conin
ttyst:		EQU a80const

